@@ -2,7 +2,6 @@ method.to.proper.method <- function(method) {
   method <- tolower(method)
   if      (method %in% c("ps")) return("ps")
   else if (method %in% c("gbm", "gbr")) return("gbm")
-  else if (method %in% c("twang")) return("twang")
   else if (method %in% c("cbps", "cbgps")) return("cbps")
   else if (method %in% c("npcbps", "npcbgps")) return("npcbps")
   else if (method %in% c("entropy", "ebal", "ebalance")) return("ebal")
@@ -18,7 +17,6 @@ check.acceptable.method <- function(method, msm = FALSE, force = FALSE) {
   bad.method <- FALSE
   acceptable.methods <- c("ps"
                           , "gbm", "gbr"
-                          , "twang"
                           , "cbps", "cbgps"
                           , "npcbps", "npcbgps"
                           , "ebal", "entropy", "ebalance"
@@ -38,7 +36,10 @@ check.acceptable.method <- function(method, msm = FALSE, force = FALSE) {
   }
   else if (!is.function(method)) bad.method <- TRUE
 
-  if (bad.method) stop("'method' must be a string of length 1 containing the name of an acceptable weighting method or a function that produces weights.", call. = FALSE)
+  if (bad.method) {
+    if (identical(method, "twang")) stop('"twang" is no longer an acceptable argument to \'method\'. Please use "gmb" for generalized boosted modeling.', call. = FALSE)
+    stop("'method' must be a string of length 1 containing the name of an acceptable weighting method or a function that produces weights.", call. = FALSE)
+  }
 
   if (msm && !force && is.character(method)) {
     m <- method.to.proper.method(method)
@@ -64,7 +65,6 @@ method.to.phrase <- function(method) {
     method <- method.to.proper.method(method)
     if (method %in% c("ps")) return("propensity score weighting")
     else if (method %in% c("gbm")) return("propensity score weighting with GBM")
-    else if (method %in% c("twang")) return("generalized boosted modeling with TWANG")
     else if (method %in% c("cbps")) return("covariate balancing propensity score weighting")
     else if (method %in% c("npcbps")) return("non-parametric covariate balancing propensity score weighting")
     else if (method %in% c("ebal")) return("entropy balancing")
@@ -82,7 +82,6 @@ process.estimand <- function(estimand, method, treat.type) {
   AE <- list(
     binary = list(ps = c("ATT", "ATC", "ATE", "ATO", "ATM", "ATOS")
                   , gbm = c("ATT", "ATC", "ATE", "ATO", "ATM")
-                  , twang = c("ATT", "ATC", "ATE")
                   , cbps = c("ATT", "ATC", "ATE")
                   , npcbps = c("ATE")
                   , ebal = c("ATT", "ATC", "ATE")
@@ -95,7 +94,6 @@ process.estimand <- function(estimand, method, treat.type) {
     ),
     multinomial = list(ps = c("ATT", "ATC", "ATE", "ATO", "ATM")
                        , gbm = c("ATT", "ATC", "ATE", "ATO", "ATM")
-                       , twang = c("ATT", "ATC", "ATE")
                        , cbps = c("ATT", "ATC", "ATE")
                        , npcbps = c("ATE")
                        , ebal = c("ATT", "ATC", "ATE")
@@ -122,7 +120,6 @@ check.subclass <- function(method, treat.type) {
   AE <- list(
     binary = list(ps = TRUE
                   , gbm = TRUE
-                  , twang = FALSE
                   , cbps = TRUE
                   , npcbps = FALSE
                   , ebal = FALSE
@@ -135,7 +132,6 @@ check.subclass <- function(method, treat.type) {
     ),
     multinomial = list(ps = TRUE
                        , gbm = TRUE
-                       , twang = FALSE
                        , cbps = FALSE
                        , npcbps = FALSE
                        , ebal = FALSE
@@ -320,7 +316,7 @@ process.by <- function(by, data, treat, treat.name = NULL, by.arg = "by") {
   return(by.components)
 }
 process.moments.int <- function(moments, int, method) {
-  # if (!is.function(method)) {
+
   if (is.function(method) || method %in% c("npcbps", "ebal", "ebcw", "optweight", "energy")) {
     if (length(int) != 1 || !is.logical(int)) {
       stop("int must be a logical (TRUE/FALSE) of length 1.", call. = FALSE)
@@ -349,9 +345,7 @@ process.moments.int <- function(moments, int, method) {
     int <- FALSE
   }
   moments <- as.integer(moments)
-  # }
-  # else {
-  # }
+
   return(list(moments = moments, int = int))
 }
 process.MSM.method <- function(is.MSM.method, method) {
@@ -381,7 +375,6 @@ process.missing <- function(missing, method, treat.type) {
                                   # , "saem"
   )
   , gbm = c("ind", "surr")
-  , twang = c("ind", "surr")
   , cbps = c("ind")
   , npcbps = c("ind")
   , ebal = c("ind")
@@ -394,7 +387,6 @@ process.missing <- function(missing, method, treat.type) {
   ),
   multinomial = list(ps = c("ind")
                      , gbm = c("ind", "surr")
-                     , twang = c("ind", "surr")
                      , cbps = c("ind")
                      , npcbps = c("ind")
                      , ebal = c("ind")
@@ -409,7 +401,6 @@ process.missing <- function(missing, method, treat.type) {
                            # , "saem"
   )
   , gbm = c("ind", "surr")
-  , twang = c("ind", "surr")
   , cbps = c("ind")
   , npcbps = c("ind")
   , ebal = c("ind")
@@ -534,114 +525,6 @@ int.poly.f <- function(d, ex = NULL, int = FALSE, poly = 1, center = TRUE, ortho
 
   return(out)
 }
-.get.s.d.denom.weightit <- function(s.d.denom = NULL, estimand = NULL, weights = NULL, treat = NULL, focal = NULL) {
-  check.estimand <- check.weights <- check.focal <- bad.s.d.denom <- bad.estimand <- FALSE
-  s.d.denom.specified <- is_not_null(s.d.denom)
-  estimand.specified <- is_not_null(estimand)
-
-  if (is_not_null(weights) && !is.data.frame(weights)) weights <- data.frame(weights)
-
-  if (s.d.denom.specified) {
-    try.s.d.denom <- tryCatch(match_arg(s.d.denom, c("treated", "control", "pooled", "all"), several.ok = TRUE),
-                              error = function(cond) FALSE)
-    if (any(try.s.d.denom == FALSE)) {
-      check.estimand <- TRUE
-      bad.s.d.denom <- TRUE
-    }
-    else {
-      if (length(try.s.d.denom) > 1 && length(try.s.d.denom) != ncol(weights)) {
-        stop("'s.d.denom' must have length 1 or equal to the number of valid sets of weights.", call. = FALSE)
-      }
-      else s.d.denom <- try.s.d.denom
-    }
-  }
-  else {
-    check.estimand <- TRUE
-  }
-
-  if (check.estimand == TRUE) {
-    if (estimand.specified) {
-      try.estimand <- tryCatch(match_arg(tolower(estimand), c("att", "atc", "ate"), several.ok = TRUE),
-                               error = function(cond) FALSE)
-      if (any(try.estimand == FALSE)) {
-        check.focal <- TRUE
-        bad.estimand <- TRUE
-      }
-      else {
-        if (length(try.estimand) > 1 && length(try.estimand) != ncol(weights)) {
-          stop("'estimand' must have length 1 or equal to the number of valid sets of weights.", call. = FALSE)
-        }
-        else s.d.denom <- vapply(try.estimand, switch, character(1L), att = "treated", atc = "control", ate = "pooled")
-      }
-    }
-    else {
-      check.focal <- TRUE
-    }
-  }
-  if (check.focal == TRUE) {
-    if (is_not_null(focal)) {
-      s.d.denom <- "treated"
-      estimand <- "att"
-    }
-    else check.weights <- TRUE
-  }
-  if (check.weights) {
-    if (is_null(weights)) {
-      s.d.denom <- "pooled"
-      estimand <- "ate"
-    }
-    else {
-      s.d.denom <- estimand <- character(ncol(weights))
-      for (i in seq_col(weights)) {
-        if (is_binary(treat)) {
-          if (all_the_same(weights[[i]][treat==1 & !check_if_zero(weights[[i]])]) &&
-              !all_the_same(weights[[i]][treat==0 & !check_if_zero(weights[[i]])])
-          ) { #if treated weights are the same and control weights differ; ATT
-            estimand[i] <- "att"
-            s.d.denom[i] <- "treated"
-          }
-          else if (all_the_same(weights[[i]][treat==0 & !check_if_zero(weights[[i]])]) &&
-                   !all_the_same(weights[[i]][treat==1 & !check_if_zero(weights[[i]])])
-          ) { #if control weights are the same and treated weights differ; ATC
-            estimand[i] <- "atc"
-            s.d.denom[i] <- "control"
-          }
-          else {
-            estimand[i] <- "ate"
-            s.d.denom[i] <- "pooled"
-          }
-        }
-        else {
-          if (length(focal) == 1) {
-            estimand[i] <- "att"
-            s.d.denom[i] <- "treated"
-          }
-          else {
-            estimand[i] <- "ate"
-            s.d.denom[i] <- "pooled"
-          }
-        }
-      }
-    }
-  }
-  if (is_not_null(weights) && length(s.d.denom) == 1) s.d.denom <- rep(s.d.denom, ncol(weights))
-
-  if (s.d.denom.specified && bad.s.d.denom && (!estimand.specified || bad.estimand)) {
-    # message("Warning: s.d.denom should be one of \"treated\", \"control\", \"pooled\", or \"all\".\n         Using \"", word_list(s.d.denom), "\" instead.")
-  }
-  else if (estimand.specified && bad.estimand) {
-    # message("Warning: the supplied estimand is not allowed. Using \"", ifelse(all_the_same(estimand), toupper(estimand)[1], word_list(toupper(estimand))), "\" instead.")
-  }
-  else if (check.focal || check.weights) {
-    # message("Note: estimand not specified; assuming ", ifelse(all_the_same(toupper(estimand)), toupper(unique(estimand)), word_list(toupper(estimand))), ".")
-  }
-
-  if (is_not_null(weights) && length(s.d.denom) != ncol(weights)) {
-    # stop("Valid inputs to s.d.denom or estimand must have length 1 or equal to the number of valid sets of weights.", call. = FALSE)
-  }
-
-  return(s.d.denom)
-}
 get.s.d.denom.weightit <- function(s.d.denom = NULL, estimand = NULL, weights = NULL, treat = NULL, focal = NULL) {
   check.estimand <- check.weights <- check.focal <- FALSE
   s.d.denom.specified <- is_not_null(s.d.denom)
@@ -834,7 +717,41 @@ compute_s.d.denom <- function(mat, treat, s.d.denom = "pooled", s.weights = NULL
   }
   return(denoms)
 }
+check_estimated_weights <- function(w, treat, treat.type, s.weights) {
 
+  tw <- w*s.weights
+
+  extreme.warn <- FALSE
+  if (treat.type == "continuous") {
+    if (all_the_same(w)) {
+      warning(paste0("All weights are ", w[1], ", possibly indicating an estimation failure."), call. = FALSE)
+    }
+    else if (sd(tw, na.rm = TRUE)/mean(tw, na.rm = TRUE) > 4) extreme.warn <- TRUE
+  }
+  else {
+    if (all_the_same(w)) {
+      warning(paste0("All weights are ", w[1], ", possibly indicating an estimation failure."), call. = FALSE)
+    }
+    else {
+      t.levels <- unique(treat)
+      bad.treat.groups <- setNames(rep(FALSE, length(t.levels)), t.levels)
+      for (i in t.levels) {
+        ti <- which(treat == i)
+        if (all(is.na(w[ti])) || all(w[ti] == 0)) bad.treat.groups[as.character(i)] <- TRUE
+        else if (!extreme.warn && sum(!is.na(tw[ti])) > 1 && sd(tw[ti], na.rm = TRUE)/mean(tw[ti], na.rm = TRUE) > 4) extreme.warn <- TRUE
+      }
+
+      if (any(bad.treat.groups)) {
+        n <- sum(bad.treat.groups)
+        warning(paste0("All weights are NA or 0 in treatment ", ngettext(n, "group ", "groups "),
+                       word_list(t.levels[bad.treat.groups], quotes = TRUE), "."), call. = FALSE)
+      }
+    }
+  }
+
+  if (extreme.warn) warning("Some extreme weights were generated. Examine them with summary() and maybe trim them with trim().", call. = FALSE)
+
+}
 ps_to_ps_mat <- function(ps, treat, assumed.treated = NULL, treat.type = NULL, treated = NULL, estimand = NULL) {
   if (is_(ps, c("matrix", "data.frame"))) {
     ps.names <- rownames(ps)
